import Asm4_libs as Asm4
import FreeCAD
import FreeCADGui as Gui
import math
from .Base import Connector, ConnectionManager

TO_RAD = 180 / math.pi

class Asm4ConnectionManager(ConnectionManager):
    root_assembly = None
    parts = None
    lcs_origin = None

    def __init__(self, *args, **kwargs):

        super().__init__(*args, **kwargs)

    def assemble(self):
        for ref in self.connections_by_ref:
            connection = self.connections_by_ref[ref]
            connection.attach()

    def create_assembly(self):
        Gui.runCommand('Asm4_newAssembly', 0)
        self.root_assembly = self.doc.getObject('Assembly')
        self.lcs_origin = self.root_assembly.getObject('LCS_Origin')
        self.parts = self.doc.getObject('Parts')

class Asm4Connector(Connector):
    def __init__(self, *args, **kargs):
        self.lcs = None

        super().__init__(*args, **kargs)

        self.create_lcs()
        self.create_link()

    def attach(self):
        if not self.connected_to: return

        print(self.reference, self.connected_to)
        connected_to = self.conn_man.get_connection_by_ref(self.connected_to)
        connected_to_board = self.conn_man.board_man.get_board_by_connection_ref(self.connected_to)

        if not connected_to or not connected_to_board: return

        print(connected_to, connected_to_board)
        Asm4.makeAsmProperties(self.parent)

        if self.connected_to == 'root':
            a_lcs = self.conn_man.lcs_origin.Name
            a_link = 'Parent Assembly'
            a_part = None
            l_part = self.doc.Name
        else:
            a_lcs = connected_to.lcs.Name
            print(a_lcs)
            a_link = connected_to_board.part.Name
            a_part = self.doc.Name
            l_part = self.doc.Name

        self.link.AttachedBy = f'#{self.lcs.Name}'
        self.link.AttachedTo = f'{a_link}#{a_lcs}'
        self.link.SolverId = 'Asm4EE'

        expr = f'{connected_to_board.part.Name}.Placement * {connected_to.lcs.Name}.Placement * AttachmentOffset * {self.lcs.Name}.Placement ^ -1'
        print(expr)
        self.link.setExpression('Placement', expr)

    def create_lcs(self):
        pos = self.footprint.GetPosition()
        ref = self.footprint.GetFieldText('Reference')
        rot = self.footprint.GetOrientationDegrees()

        lcs = self.doc.addObject('PartDesign::CoordinateSystem', f'LCS_{ref}')
        lcs.Visibility = False
        lcs.Placement.Base = FreeCAD.Vector(pos[0], -pos[1]) / 1000000.0

        self.lcs = lcs

        if self.parent:
            self.parent.addObject(lcs)

        if rot != 0:
            print('>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>', rot)
            rotation = FreeCAD.Rotation()
            rotation.Axis = (0, 0, 1)
            rotation.Angle = float(math.pi / (180 / rot))
            print(rotation)
            print(rotation.Angle)

            lcs.Placement.Rotation *= rotation
        
        if not self.board.body.Shape.isInside(lcs.Placement.Base, 0.001, True):
            rotation = FreeCAD.Rotation()
            rotation.Axis = (0, 1, 0)
            rotation.Angle = math.pi / 2

            lcs.Placement.Rotation *= rotation

    def create_link(self):
        if self.connected_to:
            connected_to = self.conn_man.get_connection_by_ref(self.connected_to)

            self.link = self.conn_man.root_assembly.newObject('App::Link', f'{self.board.group_name}_{self.reference}_{self.connected_to}')
            self.link.LinkedObject = self.parent

