import FreeCAD

class ConnectionManager:
    def __init__(self, doc, board_man=None):
        self.doc = doc
        self.board_man = board_man
        self.connections_by_ref = {}

    def add_connection_by_ref(self, ref, connection):
        self.connections_by_ref[ref] = connection
        print(f'adding {ref}', connection)

    def get_connection_by_ref(self, ref):
        return self.connections_by_ref[ref] if ref in self.connections_by_ref else None

    def assemble(self):
        raise NotImplementedError()

    def create_assembly(self):
        raise NotImplementedError()

class Connector:
    board = None

    def __init__(self, footprint, doc, parent=None, conn_man=None, board=None):
        self.board = board
        self.conn_man = conn_man
        self.connected_to = footprint.GetFieldText('Connection')
        self.connection_from = None
        self.doc = doc
        # footprint is pcbnew.FOOTPRINT
        self.footprint = footprint
        self.group = None
        self.link = None
        self.parent = parent
        self.reference = footprint.GetFieldText('Reference')
        # connections can be to, from or both

        if self.conn_man:
            self.conn_man.add_connection_by_ref(self.reference, self)

    def assemble(self):
        raise NotImplementedError()

    def __str__(self):
        if self.connection_from:
            return f'Connector: {self.reference} <- {self.connection_from}'
        else:
            return f'Connector: {self.reference} -> {self.connected_to}'

